@echo off

::::::::::::::::::::::
:: dispatch section ::
::::::::::::::::::::::

:: 
:: NOTE:
:: This batch file could have been (and was!) much more concise,
:: and also totally unreadable (meaning, more totally than now)...
:: The current structure is admittedly overkill, but it is an
:: attempt to make it easier to understand and maintain for 
:: regular humans, including me in a few days' time...
:: 

:: First, test if AUTOLINK.BAT is being called by NBMC.BAT or itself
:: (NBMC_SAR.BAT is a temp auxiliary file created by NBMC.BAT
::  before it calls AUTOLINK.BAT and deleted after the call returns)
:: 
if not exist nbmc_sar.bat goto err_msg

:: If first parameter is "-checktags" then AUTOLINK.BAT is being called by
:: NBMC.BAT to see if any {updatelink} tag(s) need(s) processing
:: in the NBMC.HTM just generated by NBMC.BAT
:: 
if %1)==-checktags) goto check_tags

:: If first parameter is "-autolinks" then AUTOLINK.BAT is being called by
:: itself to process an {autolinks} tag in NBMC.HTM during the
:: "check_tags" process (see relevant section below)
:: 
if %1)==-autolinks) goto auto_links

:: If first parameter is "-addlink" then AUTOLINK.BAT is being called by
:: itself to process an individual configuration file during the
:: "auto_links" process (see relevant section below)
:: 
if %1)==-addlink) goto add_link


::::::::::::::::::::::::
:: check_tags section ::
::::::::::::::::::::::::

:check_tags

echo. autolink: running version 1.0

:: This section is executed first when AUTOLINKS is called
:: by NBMC.BAT. Here we test for the presence of one or
:: more {autolinks} tags in NBMC.HTM, using well-
:: known old DOS batch tricks (that are documented in code
:: section below if you've never seen them).
:: Upon finding one or more of these tags AUTOLINK invokes itself
:: with the '-autolink' switch for processing of the tag by
:: the corresponding code section.
:: Note that nothing here prevents you from having the tag
:: appear several times in the layout, on top and bottom
:: of page for instance.
:: 

echo. autolink: checking for {autolinks} tag(s) in NBMC.HTM
 type %2 | find "{autolinks}" | find /v ":: " > al_test1.tmp
 copy al_test1.tmp al_test2.tmp >NUL
 if exist al_test2.tmp call autolink -autolinks %2

echo. autolink: done.
goto end


::::::::::::::::::::::
:: add_link section ::
::::::::::::::::::::::

:add_link

echo. autolink: adding link to configuration file %2
 
:: This code section is called once from "auto_links"
:: for each configuration file in the NBMC directory.
:: %1 is '-addlink', %2 is the file name.
:: 
:: First we need to determine if that configuration file contains
:: a description (ie, a non-commented {desc} tag followed by text)
:: 
 del *.tmp
 type %2 | find "{desc}" | find /v ":: " > confdesc.tmp
 copy confdesc.tmp linkname.tmp >NUL

:: If there was a description then confdesc.tmp is not empty
:: so it is copied over to linkname.tmp and we proceed to :skip1
:: 
 if exist linkname.tmp goto skip1
 
:: If there was no description, confdesk.tmp is empty:
:: DOS does not copy empty files so linkname.tmp does not
:: exist. Here we create one using the file name passed as
:: a parameter, then removing the .BMC extension
:: 
    echo %2> linkname.tmp
    gsar -i -s.bmc -r -o linkname.tmp >NUL

:skip1

:: remove the {desc} tag, if present
:: 
 gsar -s{desc}   -r  -o linkname.tmp >NUL

:: The next few lines do the actual job. Using the description
:: in linkname.tmp and GSAR commands, we create a batch file
:: named ADD_LINK.BAT that itself is a GSAR command. Its goal
:: is to find an {autolinks} tag in NBMC.HTM and replace it with
:: a link to the configuration file displaying its description, 
:: followed by a {sep} tag, a newline and an {autolinks} tag.
:: The original {autolinks} becomes something like:
:: 
:: <A HREF="filename.bmc">description</A>{sep}
:: {autolinks}
:: 
:: The ADD_LINK.BAT file itself looks something like:
:: gsar -s{autolinks} -r:060a:032href="%2":062description:060/a:062{sep}:013:010{autolinks} -o nbmc.htm>NUL
:: 
:: ...so, **again** :
:: ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Keep descriptions short, with no spaces or DOS reserved characters!
:: ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: 
 echo gsar -s{autolinks} -r:060a:032href="%2":062> add_link.bat
 gsar -s:013:010 -r  -o add_link.bat >NUL
 type linkname.tmp >> add_link.bat
 gsar -s:013:010  -r::060/a::062{sep}::013::010{autolinks}:032-o:032nbmc.htm  -o add_link.bat >NUL
 gsar -s.htm      -r.htm:062NUL:013:010                                       -o add_link.bat >NUL
 call add_link.bat

goto end


::::::::::::::::::::::::
:: auto_links section ::
::::::::::::::::::::::::

:auto_links

echo. autolink: creating automatic configuration menu

:: For each configuration file in the NBMC directory, AUTOLINK calls
:: itself with "-addlink" as the first parameter and the file name as
:: the second. See "add_link" section above.
:: Note: this means that links in the configuration menu will appear in
:: the same order as the files in the directory would in an unsorted DIR.
:: 
 for %%f in (*.bmc) do call autolink -addlink  %%f

:: Arriving here means that AUTOLINK has already processed all configuration
:: files one by one by calling itself repeatedly as above. The original
:: {autolinks} tag in NBMC.HTM has now been replaced by several lines
:: containing the description (or filename) of the configuration files,
:: each linked to the actual file and followed by a {sep} tag, as in:
:: <A HREF="filename.bmc">description</A>{sep}
:: There is a trailing {autolinks} tag following these lines.
:: 

echo. autolink: inserting link separators
 
:: Remove the trailing separator and autolinks tags
:: 
 gsar -s{sep}:013:010{autolinks}  -r        -o nbmc.htm >NUL
 
:: Replace the separator tags by some adequate string
:: the default of "&#183;" can be user-customized below
:: 
 gsar -s{sep}                     -r&#183;  -o nbmc.htm  >NUL

:: The rest of this section is mostly devoted to finding the link
:: in the menu that points to the configuration file currently
:: being used to generate NBMC.HTM, ie the parameter passed to
:: NBMC.BAT, and highlighting it in some way or other.
:: (Note: NBMC.BAT passes its first parameter as AUTOLINK.BAT's
::  second as it invokes it. AUTOLINK.BAT does not check for this).
:: 
echo. autolink: highlighting current configuration
 
:: Same trick as in the "add_link" section above, a bit modified:
:: We check for the presence of a description in the configuration
:: file by filtering, copying the result to another file and
:: checking if that file exists. Only here, we output the
:: '{desc} tag/description text' pair TWICE to the auxiliary file
:: CURRLINK.BAT to make the GSAR command generation easier.
:: 
 type %2 | find "{desc}" | find /v ":: " > confdesc.tmp
 copy /b confdesc.tmp+confdesc.tmp currlink.bat >NUL

 if exist currlink.bat goto skip2

:: If we are here then there was no description in the configuration
:: file, the output was empty and the 0-length file was not copied.
:: We then create our own using the file name as a description,
:: adding a {desc} tag and doing it twice to make the output
:: similar to the previous case. We then remove the .BMC extension.
:: 
    echo {desc}%1> currlink.bat
    echo {desc}%1>> currlink.bat
    gsar -i -s.bmc -r -o currlink.bat >NUL

:skip2
 
:: The resulting output has two lines and two {desc} tags. Here we
:: invert the second {desc} tag to {csed} and remove the middle cr/lf
:: so that only one line is left, of the form: {desc}text{csed}text
:: 
 gsar -s:013:010{desc} -r{csed} -o currlink.bat >NUL
 
:: In the following lines, the line {desc}text{csed}text becomes:
:: gsar -i -s:062text:060 -r:062{curr}text{/curr}:060 -o nbmc.htm>NUL
:: 
:: In other words, when this command is run as CURRLINK.BAT, the
:: string(s) '>text<' will be changed to '>{curr}text{/curr}<' in
:: NBMC.HTM. This is the description of the link to the current
:: configuration file; by giving values to {curr} and {/curr} we can
:: decide how we want it highlighted.
:: 
 gsar -s{desc}    -rgsar:032-i:032-s::062                   -o currlink.bat >NUL
 gsar -s{csed}    -r::060:032-r::062{curr}                  -o currlink.bat >NUL
 gsar -s:013:010  -r{/curr}::060:032-o:032nbmc.htm:013:010  -o currlink.bat >NUL
 gsar -s.htm      -r.htm:062NUL                             -o currlink.bat >NUL
 
:: Here we replace {curr} and {/curr} by meaningful strings to
:: highlight the link. This extra step is only here to make it
:: easier for users to customize the link menu's appearance.
:: 
 gsar -s{curr}    -r(                                       -o currlink.bat >NUL
 gsar -s{/curr}   -r)                                       -o currlink.bat >NUL

:: For instance, if you prefer having the text in bold type instead
:: of between parentheses, you could comment out the two lines above
:: and uncomment in the two following:
:: 
::  gsar -s{curr}    -r:060b:062                            -o currlink.bat >NUL
::  gsar -s{/curr}   -r:060/b:062                           -o currlink.bat >NUL

:: do the substitutions, clean up, and return to caller
:: 
 call currlink.bat

echo. autolink: cleaning up
 del add_link.bat
 del currlink.bat

goto end


:::::::::::::::::::::
:: err_msg section ::
:::::::::::::::::::::

:err_msg

:: ...for curious people running the batch by hand...
:: 
echo. autolink: version 1.0 - (c) Fred Pacquier - April 1998
echo. autolink: this batch file must be run by NBMC.BAT **ONLY** !


:::::::::::::::
:: at last ! ::
:::::::::::::::

:end

